import gitlab
import json
import logging
from alerta.plugins import PluginBase, app

LOG = logging.getLogger('alerta.plugins')

ALERTA_URL = 'https://{{ alerta_domain }}'
GITLAB_URL = '{{ gitlab_issue.url }}'
GITLAB_ACCESS_TOKEN = '{{ gitlab_issue.token }}'


class GitlabIssue(PluginBase):

    name = 'alerta-gitlab'

    def __init__(self):
        self.headers = {'Private-Token': GITLAB_ACCESS_TOKEN}

    def pre_receive(self, alert):
        return alert

    def post_receive(self, alert):
        return alert

    def status_change(self, alert, status, text):
        return alert, status, text

    def take_action(self, alert, action, text, **kwargs):
        if action == 'createIssue':
            """should return internal id of external system"""

            LOG.info('Create GitLab issue for %s' % alert.environment)
            git = gitlab.Gitlab(GITLAB_URL, GITLAB_ACCESS_TOKEN, ssl_verify=True) # type: gitlab
            git.auth()
            project = git.projects.get('ansible-inventories/hosts/{}/{}'.format(alert.environment, alert.origin.split('/').pop()))

            if project:
                LOG.info('Project ID %s' % project.id)


                if 'issue_iid' not in alert.attributes:
                    # noinspection PyBroadException
                    try:
                        raw_data = json.dumps(alert.raw_data, indent=2, sort_keys=True)
                    except Exception as e:
                        LOG.info('Exception %s' % str(e))
                        raw_data = alert.raw_data
                    issue = project.issues.create({
                        'title': alert.text[0:128],
                        'description': {}/alert/{}\n\nService: {}\n\nOrigin: {}\n\n```\n{}\n```\n.format(ALERTA_URL, alert.id, alert.service, alert.origin, raw_data)
                    }) # type: gitlab.Issue
                    alert.attributes['issue_iid'] = issue.id
                    alert.attributes['gitlabUrl'] = '<a href="{}" target="_blank">Issue #{}</a>'.format(
                        issue.web_url,
                        issue.iid
                    )

        return alert, action, text
